/* *****************************************************************
 * ITS 332 Information Technology II (Networking) Lab
 * Semester 2, 2009
 * SIIT
 *
 * Client/Server Programming Lab
 * File: client.c
 * Date: 11 Jan 2010
 * Version: 1.1
 *
 * Description: 
 * Client to demonstrate TCP sockets programming. You should read the
 * server.c code as well. 
 *
 * Usage:
 * client server_ip_address server_port_number
 *
 * Acknowledgement:
 * This code is based on the examples and descriptions from the 
 * Computer Science Department, Rensselaer Polytechnic Institute at:
 * http://www.cs.rpi.edu/courses/sysprog/sockets/sock.html
 *
 * ***************************************************************** */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>

/* ================================================================= */
/* error: display an error message and exit                          */
/* ================================================================= */ 
void error(char *msg)
{
    perror(msg);
    exit(0);
}

/* ================================================================= */
/* main: connect to server, prompt for a message, send the message,  */
/* receive the ack from server and then exit                         */
/* ================================================================= */ 
int main(int argc, char *argv[])
{
    /* socket file descriptor, port number of server, and number of bytes */
    int sockfd, portno, n;
    struct sockaddr_in serv_addr; /* server address */
    /* The hostent structure defines a host computer on the Internet. It  
       contains field which describe the host name, aliases for the name,
       address type and actual address (e.g. IP address) */
    struct hostent *server; 
    char buffer[256];

    /* The user must enter two parameters on the command line:
        - server host name or IP address
        - port number used by server */
    if (argc < 3) {
       fprintf(stderr,"usage %s hostname port\n", argv[0]);
       exit(0);
    }
    /* Get the port number for server entered by user */
    portno = atoi(argv[2]);

    /* Create an Internet stream (TCP) socket */
    sockfd = socket(AF_INET, SOCK_STREAM, 0);
    if (sockfd < 0) 
        error("ERROR opening socket");

    /* The gethostbyname() system call uses DNS to determine the IP 
       address of the host */
    server = gethostbyname(argv[1]);
    if (server == NULL) {
        fprintf(stderr,"ERROR, no such host\n");
        exit(0);
    }

    /* Set the server address to all zeros */
    bzero((char *) &serv_addr, sizeof(serv_addr));
    serv_addr.sin_family = AF_INET; /* Internet family of protocols */

    /* Copy server address obtained from gethostbyname to our 
       serv_addr structure */
    bcopy((char *)server->h_addr, 
         (char *)&serv_addr.sin_addr.s_addr,
         server->h_length);

    /* Convert port number to network byte order */
    serv_addr.sin_port = htons(portno); 

    /* The connect() system call establishes a connection to the server. The
       three parameters are:
        - socket file descriptor
        - address of server
        - size of the server's address */
    if (connect(sockfd,(struct sockaddr *) &serv_addr,sizeof(serv_addr)) < 0) 
        error("ERROR connecting");

    /* Once connected, the client prompts for a message, and the users
       input message is obtained with fgets() and written to the socket
       using write(). */
    printf("Please enter the message: ");
    bzero(buffer,256);
    fgets(buffer,255,stdin);
    n = write(sockfd,buffer,strlen(buffer));
    if (n < 0) 
         error("ERROR writing to socket");

    /* Zero a buffer and then read from the socket */
    bzero(buffer,256);
    n = read(sockfd,buffer,255);
    if (n < 0) 
         error("ERROR reading from socket");

    /* Display the received message and then quit the program */
    printf("%s\n",buffer);
    return 0;
}
